/* ------------------------------------------------------------
 auto_updater_updates_v2.1.js 2018-09-27
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2018 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

/**
 * AutoUpdater updates controller
 */
(function() {
	'use strict';
	
	/**
	 * Main container
	 *
	 * @type {Object}
	 */
	const $this = $('.auto-updater-updates-container');
	
	/**
	 * URLs
	 *
	 * @type {{processUpdate: string, checkFilePermission: string, checkFtpConnection: string}}
	 */
	const urls = {
		gambioUpdater: '../gambio_updater/',
		processUpdate: 'auto_updater.php?do=processUpdate',
		checkFilePermission: 'admin.php?do=AutoUpdaterAjax/checkPermission',
		checkFtpConnection: 'admin.php?do=AutoUpdaterAjax/checkFtpConnection'
	};
	
	
	/**
	 * Several modal objects
	 *
	 * @type {{downloadUpdates: (jQuery|HTMLElement), uninstalledUpdates: (jQuery|HTMLElement), updateDetails: (jQuery|HTMLElement), info: (jQuery|HTMLElement)}}
	 */
	const $modal = {
		processUpdate: $('.processing-updates.modal'),
		uninstalledUpdates: $('.uninstalled-updates.modal'),
		updateDetails: $('.update-details.modal'),
		info: $('.info.modal'),
		dbBackup: $('div.db-backup.modal')
	};
	
	
	/**
	 * Total number of selected updates
	 *
	 * @type {int}
	 */
	let selectedUpdatesTotal = 0;
	
	
	/**
	 * Total number of currently installed updates
	 *
	 * @type {int}
	 */
	let installedUpdatesTotal = 0;
	
	
	/**
	 * Flag for update status. If its true, the current update process failed.
	 *
	 * @type {boolean}
	 */
	let updateFailed = false;
	
	
	/**
	 * Flag for update status. If its true, the current update process failed.
	 *
	 * @type {boolean}
	 */
	let filePermissionOk = true;
	
	//
	// HANDLER FUNCTIONS
	//
	
	/**
	 * Click Handler for the "go to gambio updater" action
	 *
	 * @param {MessageEvent} event Triggered event.
	 */
	function gotoGambioUpdater(event) {
		window.location.href = urls.gambioUpdater;
	}
	
	/**
	 * Click handler for the "show update details" action
	 *
	 * @param {MessageEvent} event Triggered event.
	 */
	function showUpdateDetails(event) {
		event.preventDefault();
		
		const id = $(this).data('id');
		const $updateWrapper = $this.find('#' + id);
		
		const title = $updateWrapper.find('h4.update-title').text();
		const description = $updateWrapper.find('div.update-description').html();
		
		$modal['updateDetails'].find('.modal-title').text(title);
		$modal['updateDetails'].find('.modal-body')
			.html(description)
			.prepend($updateWrapper.find('div.update-icon').clone().addClass('pull-right'));
		$modal['updateDetails'].modal('show');
	}
	
	/**
	 * Handles the change events, that are triggered if an update change its selection.
	 *
	 * @param {MessageEvent} event Triggered event.
	 */
	function updateTextOfStateProcessingButton(event) {
		const $button = $('input.download-updates');
		selectedUpdatesTotal = $this.find('input.update-selection:checked').size();
		
		if (selectedUpdatesTotal === 0) {
			$button.val($button.data('textPlural').replace('%s', selectedUpdatesTotal)).prop('disabled', true);
		} else if (selectedUpdatesTotal > 1) {
			$button.val($button.data('textPlural').replace('%s', selectedUpdatesTotal)).prop('disabled', false);
		} else {
			$button.val($button.data('textSingle').replace('%s', selectedUpdatesTotal)).prop('disabled', false);
		}
	}
	
	/**
	 * Click handler for "update selected/unselected" events
	 *
	 * @param {MessageEvent} event Triggered event.
	 */
	function changeUpdateSelection(event) {
		const $updateWrapper = $(this).closest('.update-wrapper');
		
		if ($updateWrapper.find('input.update-selection').is(':disabled')) {
			return;
		}
		
		const id = $updateWrapper.data('id');
		const isChecked = $updateWrapper.find('input.update-selection').is(':checked');
		
		$updateWrapper.find('input.update-selection').single_checkbox('checked', !isChecked);
		if (!isChecked) {
			$updateWrapper.addClass('active');
		} else {
			$updateWrapper.removeClass('active');
		}
		
		updateTextOfStateProcessingButton();
	}
	
	/**
	 * Click handler for the "download selected updates" action
	 *
	 * @param {MessageEvent} event Triggered event.
	 */
	function processUpdates(event) {
		event.preventDefault();
		
		$modal['dbBackup'].modal('hide');
		selectedUpdatesTotal = $this.find('input.update-selection:checked').length;
		installedUpdatesTotal = 0;
		
		// Check if any update has been selected, Otherwise return without action
		if (selectedUpdatesTotal === 0) {
			return;
		}
		
		let updateIds = [];
		$this.find('input.update-selection:checked').each(
			function(index, element) {
				updateIds.push($(element).val());
			}
		);
		
		resetUpdateProcessingModal();
		checkFilePermissions(updateIds);
	}
	
	/**
	 * Handler for the switch events of the ftp protocol
	 *
	 * @param {MessageEvent} event Triggered event.
	 */
	function switchedFtpProtocol(event) {
		if ($(this).val() === 'ftp') {
			$modal['processUpdate'].find('.ftp-passive').show();
			$modal['processUpdate'].find('.ftp-port').hide();
			return;
		}
		$modal['processUpdate'].find('.ftp-passive').hide();
		$modal['processUpdate'].find('.ftp-port').show();
	}
	
	/**
	 * Handles the "test ftp connection" action and starts the update processing on success
	 *
	 * @param {MessageEvent} event Triggered event.
	 */
	function testFtpConnection(event) {
		event.preventDefault();
		
		$.ajax({
			type: "POST",
			url: urls.checkFtpConnection,
			data: $modal['processUpdate'].find('.ftp-form').serialize(),
			success: function(response) {
				try {
					response = JSON.parse(response);
				} catch (e) {
					response = {
						success: false
					};
				}
				
				if (response.success === false) {
					$modal['processUpdate'].find('p.ftp-form-error').show();
					
					return;
				}
				
				activateUpdateProcessingStartedState();
				processUpdate(getNextUpdateToProcess());
			},
			error: function() {
				$modal['processUpdate'].find('p.ftp-form-error').show();
			},
		});
	}
	
	//
	// HELPER FUNCTIONS
	//
	
	/**
	 * Checks file permissions for all given update id. Will call itself recursively with one update id less.
	 */
	function checkFilePermissions(updateIds) {
		if (updateIds.length === 0 && filePermissionOk === true) {
			activateUpdateProcessingStartedState();
			processUpdate(getNextUpdateToProcess());
			
			return;
		} else if (filePermissionOk === false) {
			updateProgressBar(100, true);
			activateFilePermissionFailedState();
			
			return;
		}
		
		$.ajax({
			type: "POST",
			url: urls.checkFilePermission,
			data: {
				'updateId': updateIds.pop()
			},
			success: function(response) {
				try {
					response = JSON.parse(response);
				} catch (e) {
					response = {
						success: false,
						result: false
					};
				}
				
				updateProgressBar(Math.ceil(((selectedUpdatesTotal - updateIds.length) / selectedUpdatesTotal) * 100));
				filePermissionOk = response.success === true && response.result === true;
				checkFilePermissions(updateIds);
			},
			error: function() {
				filePermissionOk = response.success === true && response.result === true;
				checkFilePermissions([]);
			},
		});
	}
	
	/**
	 * Starts the processing for a given update
	 */
	function processUpdate(update) {
		if (update === undefined) {
			$modal['processUpdate'].find('.modal-footer').show();
			$modal['processUpdate'].find('.processing').hide();
			if (installedUpdatesTotal > 0) {
				$modal['processUpdate'].find('.close-button').on('click', gotoGambioUpdater);
			}
			
			return;
		}
		
		$modal['processUpdate'].find('h4.processing span').text(update.updateTitle);
		$modal['processUpdate'].find('ul.processing')
			.html('<li>' + jse.core.lang.translate('states_processing', 'auto_updater') + '</li>');
		
		const formdata = $modal['processUpdate'].find('.ftp-form').serializeArray();
		let data = {};
		$(formdata).each(function(index, obj) {
			data[obj.name] = obj.value;
		});
		data.selectedUpdate = update.updateId;
		
		$.ajax({
			type: "POST",
			url: urls.processUpdate,
			data: data,
			success: function(response) {
				if (response.success === false) {
					activateUpdateProcessingFailedState();
					displayUpdateProcessingFailedInformation(response.state, response.error, response.list);
					
					return;
				} else if (response.done === false) {
					updateProgressBar(Math.ceil((installedUpdatesTotal + response.progress) / selectedUpdatesTotal
						* 100));
					processUpdate(update);
					
					return;
				}
				
				displayUpdateProcessingSucceededInformation(update, response.progress);
				processUpdate(getNextUpdateToProcess());
			}
		});
	}
	
	/**
	 * Returns needed information of an update, that need to be processed next
	 */
	function getNextUpdateToProcess() {
		if ($this.find('input.update-selection:checked').first().length === 0) {
			return undefined;
		}
		
		const updateId = $this.find('input.update-selection:checked').first().val();
		const updateTitle = $this.find('#' + updateId).find('h4.update-title').text().trim();
		
		return {
			updateId: updateId,
			updateTitle: updateTitle
		};
	}
	
	/**
	 * Resets the update processing modal
	 */
	function resetUpdateProcessingModal() {
		updateFailed = false;
		$('input.download-updates').prop('disabled', true);
		
		updateProgressBar(0);
		$modal['processUpdate'].find('.modal-title')
			.text(jse.core.lang.translate('modal_download_updates_title_permission_check', 'auto_updater'));
		$modal['processUpdate'].find('.permission-check-failed').hide();
		$modal['processUpdate'].find('.processing').hide();
		$modal['processUpdate'].find('.errors').hide();
		$modal['processUpdate'].find('.succeed').hide();
		$modal['processUpdate'].find('.modal-footer .ftp-test').hide();
		$modal['processUpdate'].find('.modal-footer .close-button').hide();
		
		$modal['dbBackup'].on('hidden.bs.modal', function(e) {
			$modal['processUpdate'].modal({
				show: true,
				backdrop: "static",
				keyboard: false
			});
			$modal['dbBackup'].off();
		});
		$modal['dbBackup'].modal('hide');
	}
	
	/**
	 * Activates the file permission failed state
	 */
	function activateFilePermissionFailedState() {
		$modal['processUpdate'].find('.ftp-form-error').hide();
		$modal['processUpdate'].find('.permission-check-failed').show();
		$modal['processUpdate'].find('.modal-footer .close-button').hide();
		$modal['processUpdate'].find('.modal-footer .ftp-test').show();
		$modal['processUpdate'].find('.modal-footer').show();
		$modal['processUpdate'].find('.ftp-port').hide();
		
		$modal['processUpdate'].find('input[name=ftp-protocol]').on('change', switchedFtpProtocol);
		$modal['processUpdate'].find('.modal-footer .ftp-test').on('click', testFtpConnection);
	}
	
	/**
	 * Activates the update processing started state
	 */
	function activateUpdateProcessingStartedState() {
		updateFailed = false;
		$('input.download-updates').prop('disabled', true);
		
		updateProgressBar(0);
		$modal['processUpdate'].find('.modal-title')
			.text(jse.core.lang.translate('modal_download_updates_title_processing', 'auto_updater'));
		$modal['processUpdate'].find('.permission-check-failed').hide();
		$modal['processUpdate'].find('.permission-check').hide();
		$modal['processUpdate'].find('.processing').show();
		$modal['processUpdate'].find('.progress').show();
		$modal['processUpdate'].find('.modal-footer .close-button').show();
		$modal['processUpdate'].find('.modal-footer .ftp-test').hide();
		$modal['processUpdate'].find('.modal-footer').hide();
	}
	
	/**Activates the update processing failed state
	 */
	function activateUpdateProcessingFailedState() {
		updateFailed = true;
		updateProgressBar(100, true);
		$modal['processUpdate'].find('.progress .progress-bar').removeClass('active');
		$modal['processUpdate'].find('.modal-footer').show();
		if (installedUpdatesTotal > 0) {
			$modal['processUpdate'].find('.modal-footer').show();
			$modal['processUpdate'].find('.close-button').on('click', gotoGambioUpdater);
		}
	}
	
	/**
	 * Displays the update processing failed information
	 */
	function displayUpdateProcessingFailedInformation(state, error, fileList) {
		const $processList = $modal['processUpdate'].find('ul.processing-list');
		$processList.find('li:first').remove();
		switch (state) {
			case 'installed':
				$processList.prepend('<li>' + jse.core.lang.translate('states_update_done', 'auto_updater')
					+ '</li>');
				$processList.prepend('<li>'
					+ jse.core.lang.translate('states_update_copied', 'auto_updater') + '</li>');
			case 'backup_restore_failed':
				$processList.prepend('<li>'
					+ jse.core.lang.translate('states_backup_created', 'auto_updater') + '</li>');
			case 'backup_creation_failed':
				$processList.prepend('<li>'
					+ jse.core.lang.translate('states_file_check_done', 'auto_updater') + '</li>');
			case 'permission_check_failed':
				$processList.prepend('<li>'
					+ jse.core.lang.translate('states_download_done', 'auto_updater') + '</li>');
		}
		$processList.append('<li><b>' + jse.core.lang.translate('states_error', 'auto_updater') + ':</b> '
			+ jse.core.lang.translate(error, 'auto_updater') + '</li>');
		
		if (state === 'permission_check_failed') {
			$processList.append(fileList);
		}
	}
	
	/**
	 * Displays the update processing succeeded information
	 */
	function displayUpdateProcessingSucceededInformation(update, progress) {
		installedUpdatesTotal++;
		updateProgressBar(Math.ceil(((installedUpdatesTotal + progress) / selectedUpdatesTotal) * 100));
		$modal['processUpdate'].find('.succeed').show();
		$modal['processUpdate'].find('ul.succeed-list')
			.append('<li>' + update.updateTitle + '</li>');
		
		$this.find('#' + update.updateId)
			.find('input.update-selection')
			.single_checkbox('checked', false)
			.trigger('change');
	}
	
	/**
	 * Updates the progress bar
	 *
	 * @param {number} progress  Current progress in percent.
	 * @param {boolean} canceled  True, if the job was canceled.
	 */
	function updateProgressBar(progress, canceled = false) {
		const $progressBar = $modal['processUpdate'].find('.progress .progress-bar');
		
		if (!Number.isInteger(progress) || progress < 0) {
			progress = 0;
		} else if (progress > 100) {
			progress = 100;
		}
		
		$progressBar.prop('aria-valuenow', progress);
		$progressBar.css('width', progress + '%');
		$progressBar.text(progress + '%');
		
		if (canceled) {
			$progressBar.removeClass('active');
			$progressBar.addClass('progress-bar-danger');
			return;
		}
		
		$progressBar.removeClass('progress-bar-danger');
		$progressBar.addClass('active');
		if (progress === 100) {
			$progressBar.removeClass('active');
		}
	}
	
	//
	// INITIALISATION
	//
	
	/**
	 * Handles the module initialization.
	 */
	function onInit() {
		$this.find('a.update-details').on('click', showUpdateDetails);
		
		$this.find('input.update-selection').on('change', updateTextOfStateProcessingButton);
		
		$this.find('h4.update-title').on('click', changeUpdateSelection);
		$this.find('div.update-icon').on('click', changeUpdateSelection);
		$this.find('div.update-description-short').on('click', changeUpdateSelection);
		
		$modal['dbBackup'].find('button.process-updates').on('click', processUpdates);
		
		$this.find('.tooltip-icon').on('click', () => {
			$modal['info'].modal('show')
		});
		
		if ($this.data('showInfo') === true) {
			$modal['info'].modal({
				show: true,
				backdrop: "static",
				keyboard: false
			});
		} else if ($modal['uninstalledUpdates'].length) {
			$modal['uninstalledUpdates'].modal({
				show: true,
				backdrop: "static",
				keyboard: false
			});
		}
	}
	
	document.addEventListener('DOMContentLoaded', onInit);
})();